@@ -4,4 +4,7 @@ USER_HOME_ELEANOR_CACHE = os.path.join(os.path.expanduser('~'), '.eleanor/')
 USER_HOME_LIGHTKURVE = os.path.join(os.path.expanduser('~'), '.lightkurve-cache/')
 LIGHTKURVE_CACHE_DIR = ".lightkurve-cache/"
 ELEANOR_CACHE_DIR = ".eleanor/"
-CUTOUT_SIZE = 11
\ No newline at end of file
+CUTOUT_SIZE = 13
+MISSION_KEPLER = "Kepler"
+MISSION_K2 = "K2"
+MISSION_TESS = "TESS"
\ No newline at end of file
