@@ -8,10 +8,15 @@ import numpy
 import pandas
 import yaml
 import multiprocessing
+
+from statsmodels.tsa.stattools import acf
+
+from lcbuilder.constants import ELEANOR_CACHE_DIR, LIGHTKURVE_CACHE_DIR
 from scipy import stats
 from scipy.signal import savgol_filter
 from wotan import flatten
 
+from lcbuilder import constants
 from lcbuilder.helper import LcbuilderHelper
 from lcbuilder.star.starinfo import StarInfo
 
@@ -30,7 +35,8 @@ import foldedleastsquares as tls
 
 class LcBuilder:
     COORDS_REGEX = "^(-{0,1}[0-9.]+)_(-{0,1}[0-9.]+)$"
-    DEFAULT_CADENCES_FOR_MISSION = {"Kepler": 60, "K2": 60, "TESS": 120}
+    DEFAULT_CADENCES_FOR_MISSION = {constants.MISSION_KEPLER: 60, constants.MISSION_K2: 60,
+                                    constants.MISSION_TESS: 120}
 
     def __init__(self) -> None:
         self.lightcurve_builders = {InputObjectInfo: MissionInputLightcurveBuilder(),
@@ -41,10 +47,19 @@ class LcBuilder:
 
     def build(self, object_info: ObjectInfo, object_dir: str, caches_root_dir=os.path.expanduser('~') + "/",
               cpus=multiprocessing.cpu_count() - 1):
+        if not os.path.exists(caches_root_dir):
+            os.mkdir(caches_root_dir)
+        lightkurve_cache_dir = caches_root_dir + LIGHTKURVE_CACHE_DIR
+        if not os.path.exists(lightkurve_cache_dir):
+            os.mkdir(lightkurve_cache_dir)
+        eleanor_cache_dir = caches_root_dir + ELEANOR_CACHE_DIR
+        if not os.path.exists(eleanor_cache_dir):
+            os.mkdir(eleanor_cache_dir)
         lc_build = self.lightcurve_builders[type(object_info)].build(object_info, object_dir, caches_root_dir)
         if lc_build.tpf_apertures is not None:
             with open(object_dir + "/apertures.yaml", 'w') as f:
-                apertures = {sector: [aperture.tolist() for aperture in apertures]
+                apertures = {int(sector): [aperture.tolist() if isinstance(aperture, numpy.ndarray) else aperture
+                                      for aperture in apertures]
                              for sector, apertures in lc_build.tpf_apertures.items()}
                 apertures = {"sectors": apertures}
                 f.write(yaml.dump(apertures, default_flow_style=True))
@@ -56,6 +71,9 @@ class LcBuilder:
         flux_err_float = lc_build.lc.flux_err.value
         lc = lightkurve.LightCurve(time=time_float, flux=flux_float, flux_err=flux_err_float)
         lc_df = pandas.DataFrame(columns=['#time', 'flux', 'flux_err'])
+        time_float = numpy.array(time_float)
+        flux_float = numpy.array(flux_float)
+        flux_err_float = numpy.array(flux_err_float)
         lc_df['#time'] = time_float
         lc_df['flux'] = flux_float
         lc_df['flux_err'] = flux_err_float
@@ -68,39 +86,19 @@ class LcBuilder:
         cadence_array = cadence_array[~numpy.isnan(cadence_array)]
         cadence_array = cadence_array[cadence_array > 0]
         lc_build.cadence = int(numpy.round(numpy.nanmedian(cadence_array)))
-        clean_time, flatten_flux, clean_flux_err = self.__clean_initial_flux(object_info, time_float, flux_float,
-                                                                             flux_err_float, star_info,
-                                                                             lc_build.cadence, object_dir)
-        lc = lightkurve.LightCurve(time=clean_time, flux=flatten_flux, flux_err=clean_flux_err)
-        periodogram = self.__plot_periodogram(lc, 0.05, 15, 10, sherlock_id, 
+        lc = self.__clean_initial_flux(object_info, time_float, flux_float, flux_err_float, star_info,
+                                       lc_build.cadence, object_dir)
+        clean_time = lc.time.value
+        flatten_flux = lc.flux.value
+        clean_flux_err = lc.flux_err.value
+        periodogram = self.__plot_periodogram(lc, 0.05, 15, 10, sherlock_id,
                                               object_dir + "/Periodogram_Initial_" + str(sherlock_id) + ".png")
+        self.__plot_autocorrelation(lc, lc_build.cadence, sherlock_id,
+                                    object_dir + "/Autocorrelation_Initial_" + str(sherlock_id) + ".png")
         if object_info.auto_detrend_period is not None:
             lc_build.detrend_period = object_info.auto_detrend_period
         elif object_info.auto_detrend_enabled:
             lc_build.detrend_period = self.__calculate_max_significant_period(lc, periodogram)
-        if object_info.reduce_simple_oscillations:
-            logging.info('================================================')
-            logging.info('STELLAR OSCILLATIONS REDUCTION')
-            logging.info('================================================')
-            flatten_flux = self.__reduce_simple_oscillations(object_dir, object_info.mission_id(), clean_time,
-                                                             flatten_flux, star_info,
-                                                             object_info.oscillation_snr_threshold,
-                                                             object_info.oscillation_amplitude_threshold,
-                                                             object_info.oscillation_ws_scale,
-                                                             object_info.oscillation_min_period,
-                                                             cpus)
-        if lc_build.detrend_period is not None:
-            logging.info('================================================')
-            logging.info('AUTO-DETREND EXECUTION')
-            logging.info('================================================')
-            logging.info("Period = %.3f", lc_build.detrend_period)
-            lc.fold(lc_build.detrend_period).scatter()
-            plt.title("Phase-folded period: " + format(lc_build.detrend_period, ".2f") + " days")
-            plt.savefig(object_dir + "/Phase_detrend_period_" + str(sherlock_id) + "_" +
-                        format(lc_build.detrend_period, ".2f") + "_days.png", bbox_inches='tight')
-            plt.clf()
-            flatten_flux, lc_trend = self.__detrend_by_period(object_info.auto_detrend_method, clean_time, flatten_flux,
-                                                              lc_build.detrend_period * object_info.auto_detrend_ratio)
         if object_info.initial_mask is not None:
             logging.info('================================================')
             logging.info('INITIAL MASKING')
@@ -127,12 +125,50 @@ class LcBuilder:
                 clean_time = clean_time[~mask]
                 flatten_flux = flatten_flux[~mask]
                 clean_flux_err = clean_flux_err[~mask]
+        if object_info.reduce_simple_oscillations:
+            logging.info('================================================')
+            logging.info('STELLAR OSCILLATIONS REDUCTION')
+            logging.info('================================================')
+            flatten_flux = self.__reduce_simple_oscillations(object_dir, object_info.mission_id(), clean_time,
+                                                             flatten_flux, star_info,
+                                                             object_info.oscillation_snr_threshold,
+                                                             object_info.oscillation_amplitude_threshold,
+                                                             object_info.oscillation_ws_scale,
+                                                             object_info.oscillation_min_period,
+                                                             cpus)
+        if lc_build.detrend_period is not None:
+            logging.info('================================================')
+            logging.info('AUTO-DETREND EXECUTION')
+            logging.info('================================================')
+            logging.info("Period = %.3f", lc_build.detrend_period)
+            lc.fold(lc_build.detrend_period).scatter()
+            plt.title("Phase-folded period: " + format(lc_build.detrend_period, ".2f") + " days")
+            plt.savefig(object_dir + "/Phase_detrend_period_" + str(sherlock_id) + "_" +
+                        format(lc_build.detrend_period, ".2f") + "_days.png", bbox_inches='tight')
+            plt.clf()
+            flatten_flux, lc_trend = self.__detrend_by_period(object_info.auto_detrend_method, clean_time, flatten_flux,
+                                                              lc_build.detrend_period * object_info.auto_detrend_ratio)
         lc = lightkurve.LightCurve(time=clean_time, flux=flatten_flux, flux_err=clean_flux_err)
         lc_build.lc = lc
         self.__plot_periodogram(lc, 0.05, 15, 10, sherlock_id,
                                 object_dir + "/Periodogram_Final_" + str(sherlock_id) + ".png")
+        self.__plot_autocorrelation(lc, lc_build.cadence, sherlock_id,
+                                    object_dir + "/Autocorrelation_Final_" + str(sherlock_id) + ".png")
         return lc_build
 
+    def __plot_autocorrelation(self, lc, cadence_s, object_id, filename, max_days=5):
+        nlags = int(max_days / (cadence_s / 60 / 60 / 24))
+        correlation = acf(lc.flux.value, nlags=nlags)
+        lags = numpy.linspace(0, max_days, len(correlation))
+        fig, axs = plt.subplots(1, 1, figsize=(8, 4), constrained_layout=True)
+        axs.set_ylabel("Autocorrelation")
+        axs.set_xlabel("Time lags (d)")
+        axs.set_title(object_id + " Autocorrelation")
+        axs.plot(lags, correlation, color="blue")
+        plt.savefig(filename)
+        plt.clf()
+        plt.close()
+
     def __plot_periodogram(self, lc, min_period, max_period, oversample, object_id, filename):
         periodogram = lc.to_periodogram(minimum_period=min_period, maximum_period=max_period,
                                         oversample_factor=oversample)
@@ -151,20 +187,20 @@ class LcBuilder:
         # plt.clf()
         return periodogram
 
-    def smooth(self, flux, window_len=11, window='blackman'):
-        clean_flux = savgol_filter(flux, window_len, 3)
-        # if window_len < 3:
+    def smooth(self, flux, sg_window_len=11, convolve_window_len=7, window='blackman'):
+        # if convolve_window_len < 3 or sg_window_len < 3:
         #     return flux
         # if not window in ['flat', 'hanning', 'hamming', 'bartlett', 'blackman']:
         #     raise ValueError("Window is on of 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'")
-        # s = np.r_[np.ones(window_len//2), clean_flux, np.ones(window_len//2)]
+        # s = numpy.r_[numpy.ones(convolve_window_len // 2), flux, numpy.ones(convolve_window_len // 2)]
         # # print(len(s))
         # if window == 'flat':  # moving average
-        #     w = np.ones(window_len, 'd')
+        #     w = numpy.ones(convolve_window_len, 'd')
         # else:
-        #     w = eval('np.' + window + '(window_len)')
+        #     w = eval('numpy .' + window + '(window_len)')
         # # TODO probably problematic with big gaps in the middle of the curve
-        # clean_flux = np.convolve(w / w.sum(), s, mode='valid')
+        # clean_flux = numpy.convolve(w / w.sum(), s, mode='valid')
+        clean_flux = savgol_filter(flux, sg_window_len, 3)
         return clean_flux
 
     def __calculate_max_significant_period(self, lc, periodogram):
@@ -255,12 +291,13 @@ class LcBuilder:
             model = tls.transitleastsquares(time, flux)
             transit_period_min = 0.3
             transit_period_max = 2
+            period_grid, oversampling = LcbuilderHelper.calculate_period_grid(time, transit_period_min,
+                                                                               transit_period_max, 1, star_info, 2)
             power_args = {"period_min": 0.3,
                           "period_max": 2, "n_transits_min": 2, "show_progress_bar": False,
                           "duration_grid_step": 1.15,
-                          "use_threads": cpus, "oversampling_factor": 3,
-                          "period_grid": LcbuilderHelper.calculate_period_grid(time, transit_period_min,
-                                                                               transit_period_max, 1, star_info, 2)}
+                          "use_threads": cpus, "oversampling_factor": oversampling,
+                          "period_grid": period_grid}
             if star_info.ld_coefficients is not None:
                 power_args["u"] = star_info.ld_coefficients
             power_args["R_star"] = star_info.radius
@@ -422,6 +459,7 @@ class LcBuilder:
         clean_flux = flux
         clean_flux_err = flux_err
         is_short_cadence = cadence <= 300
+        lc = lightkurve.LightCurve(time=clean_time, flux=clean_flux, flux_err=clean_flux_err)
         if (object_info.binning > 1) or (object_info.prepare_algorithm) or (is_short_cadence and object_info.smooth_enabled) or (
                 object_info.high_rms_enabled and object_info.initial_mask is None):
             logging.info('================================================')
@@ -429,17 +467,25 @@ class LcBuilder:
             logging.info('================================================')
         if object_info.binning > 1:
             bins = len(time) / object_info.binning
+            bin_edges_for_time = numpy.arange(time[0], time[-1], (cadence / 24 / 60 / 60) * object_info.binning)
             bin_means, bin_edges, binnumber = stats.binned_statistic(time, flux, statistic='mean',
-                                                                     bins=bins)
-            bin_stds, _, _ = stats.binned_statistic(time, flux, statistic='std', bins=bins)
+                                                                     bins=bin_edges_for_time)
+            bin_stds, _, _ = stats.binned_statistic(time, flux, statistic='std', bins=bin_edges_for_time)
             bin_width = (bin_edges[1] - bin_edges[0])
             bin_centers = bin_edges[1:] - bin_width / 2
             clean_time = bin_centers
             clean_flux = bin_means
             clean_flux_err = bin_stds
+            lc = lightkurve.LightCurve(time=clean_time, flux=clean_flux, flux_err=clean_flux_err)
+            lc = lc.remove_nans()
         if object_info.prepare_algorithm is not None:
-            clean_time, clean_flux, clean_flux_err = object_info.prepare_algorithm.prepare(object_info, clean_time,
-                                                                                           clean_flux, clean_flux_err)
+            clean_time, clean_flux, clean_flux_err = object_info.prepare_algorithm.prepare(
+                object_info, lc.time.value, lc.flux.value, lc.flux_err.value)
+            lc = lightkurve.LightCurve(time=clean_time, flux=clean_flux, flux_err=clean_flux_err)
+            lc = lc.remove_nans()
+        clean_time = lc.time.value
+        clean_flux = lc.flux.value
+        clean_flux_err = lc.flux_err.value
         if object_info.high_rms_enabled and object_info.initial_mask is None:
             logging.info('Masking high RMS areas by a factor of %.2f with %.1f hours binning',
                          object_info.high_rms_threshold, object_info.high_rms_bin_hours)
@@ -458,8 +504,8 @@ class LcBuilder:
                 flux_partial = clean_flux[previous_jump_index:jumpIndex]
                 before_flux_partial = before_flux[previous_jump_index:jumpIndex]
                 bins = (time_partial[len(time_partial) - 1] - time_partial[1]) * bins_per_day
-                bin_stds, bin_edges, binnumber = stats.binned_statistic(time_partial[1:], flux_partial[1:], statistic='std',
-                                                                        bins=bins)
+                bin_stds, bin_edges, binnumber = stats.binned_statistic(time_partial[1:], flux_partial[1:],
+                                                                        statistic='std', bins=bins)
                 stds_median = numpy.nanmedian(bin_stds[bin_stds > 0])
                 stds_median_array = numpy.full(len(bin_stds), stds_median)
                 rms_threshold_array = stds_median_array * object_info.high_rms_threshold
@@ -482,10 +528,12 @@ class LcBuilder:
             clean_time = clean_time[~entire_high_rms_mask]
             clean_flux = clean_flux[~entire_high_rms_mask]
             clean_flux_err = clean_flux_err[~entire_high_rms_mask]
+            lc = lightkurve.LightCurve(time=clean_time, flux=clean_flux, flux_err=clean_flux_err)
+            lc = lc.remove_nans()
         if is_short_cadence and object_info.smooth_enabled:
             # logging.info('Applying Smooth phase (savgol + weighted average)')
             logging.info('Applying Smooth phase (savgol)')
-            clean_flux = self.smooth(clean_flux)
+            clean_flux = self.smooth(lc.flux.value)
             # TODO to use convolve we need to remove the borders effect
             # clean_flux = np.convolve(clean_flux, [0.025, 0.05, 0.1, 0.155, 0.34, 0.155, 0.1, 0.05, 0.025], "same")
             # clean_flux = np.convolve(clean_flux, [0.025, 0.05, 0.1, 0.155, 0.34, 0.155, 0.1, 0.05, 0.025], "same")
@@ -493,7 +541,9 @@ class LcBuilder:
             # clean_flux[len(clean_flux) - 6: len(clean_flux) - 1] = 1
             # clean_flux = uniform_filter1d(clean_flux, 11)
             # clean_flux = self.flatten_bw(self.FlattenInput(clean_time, clean_flux, 0.02))[0]
-        return clean_time, clean_flux, clean_flux_err
+            lc = lightkurve.LightCurve(time=clean_time, flux=clean_flux, flux_err=clean_flux_err)
+            lc = lc.remove_nans()
+        return lc
 
     def __plot_rms_mask(self, object_id, rms_bin_hours, bin_centers, bin_stds, rms_threshold_array, rms_mask,
                         time, flux, filename, object_dir):
