@@ -1,5 +1,6 @@
 import logging
 import os
+import shutil
 import sys
 import lcbuilder.eleanor
 from lcbuilder import constants
@@ -40,15 +41,18 @@ class MissionFfiLightcurveBuilder(LightcurveBuilder):
         star_info = None
         if mission_prefix not in self.star_catalogs:
             raise ValueError("Wrong object id " + mission_id)
-        sectors = None if object_info.sectors == 'all' or mission != "TESS" else object_info.sectors
-        campaigns = None if object_info.sectors == 'all' or mission != "K2" else object_info.sectors
-        quarters = None if object_info.sectors == 'all' or mission != "Kepler" else object_info.sectors
+        sectors = None if object_info.sectors == 'all' or mission != constants.MISSION_TESS else object_info.sectors
+        campaigns = None if object_info.sectors == 'all' or mission != constants.MISSION_K2 else object_info.sectors
+        quarters = None if object_info.sectors == 'all' or mission != constants.MISSION_KEPLER else object_info.sectors
         apertures = {}
         tpf_search_results = lk.search_targetpixelfile(str(mission_id))
         for tpf_search_result in tpf_search_results:
             logging.info("There is data for Mission: %s, Year %.0f, Author: %s, ExpTime: %.0f",
                          tpf_search_result.mission[0], tpf_search_result.year[0], tpf_search_result.author[0],
                          tpf_search_result.exptime[0].value)
+        tpfs_dir = sherlock_dir + "/tpfs/"
+        if not os.path.exists(tpfs_dir):
+            os.mkdir(tpfs_dir)
         if mission_prefix == self.MISSION_ID_KEPLER or mission_prefix == self.MISSION_ID_KEPLER_2:
             source = "tpf"
             lcf_search_results = lk.search_lightcurvefile(str(mission_id), mission=mission, cadence=cadence,
@@ -69,7 +73,10 @@ class MissionFfiLightcurveBuilder(LightcurveBuilder):
                 logging.info("Correcting K2 motion in light curve...")
                 sectors = [lcfile.campaign for lcfile in lcf]
                 lc = SFFCorrector(lc).correct(windows=20)
+            if not os.path.exists(tpfs_dir):
+                os.mkdir(tpfs_dir)
             for tpf in tpfs:
+                shutil.copy(tpf.path, tpfs_dir + os.path.basename(tpf.path))
                 if mission_prefix == self.MISSION_ID_KEPLER:
                     sector = tpf.quarter
                 elif mission_prefix == self.MISSION_ID_KEPLER_2:
@@ -94,9 +101,7 @@ class MissionFfiLightcurveBuilder(LightcurveBuilder):
             if star[0].tic:
                 # TODO FIX star info objectid
                 logging.info("Assotiated TIC is " + str(star[0].tic))
-                tpfs = lk.search_targetpixelfile("TIC " + str(star[0].tic), mission="TESS", cadence=cadence,
-                                                 sector=sectors, quarter=quarters,
-                                                 campaign=campaigns, author="TESS-SPOC") \
+                tpfs = lk.search_tesscut("TIC " + str(star[0].tic), sector=sectors) \
                     .download_all(download_dir=caches_root_dir + LIGHTKURVE_CACHE_DIR,
                                   cutout_size=(CUTOUT_SIZE, CUTOUT_SIZE))
                 star_info = starinfo.StarInfo(object_info.sherlock_id(), *self.star_catalog.catalog_info(int(star[0].tic)))
@@ -106,6 +111,7 @@ class MissionFfiLightcurveBuilder(LightcurveBuilder):
                 data.append(datum)
                 for tpf in tpfs:
                     if tpf.sector == s.sector:
+                        shutil.copy(tpf.path, tpfs_dir + os.path.basename(tpf.path))
                         apertures[s.sector] = ApertureExtractor.from_boolean_mask(datum.aperture.astype(bool),
                                                                                   tpf.column, tpf.row)
             quality_bitmask = np.bitwise_and(data[0].quality.astype(int), 175)
@@ -119,6 +125,7 @@ class MissionFfiLightcurveBuilder(LightcurveBuilder):
                     lc = lc.append(datum.to_lightkurve(datum.pca_flux, quality_mask=quality_bitmask).remove_nans()
                                    .flatten())
                 transits_min_count = 2
+        lc = lc.remove_nans()
         return LcBuild(lc, lc_data, star_info, transits_min_count, cadence, None, sectors, source, apertures)
 
     def extract_eleanor_lc_data(selfself, eleanor_data):
