@@ -1,6 +1,9 @@
 import logging
+import shutil
+
 import numpy as np
 
+from lcbuilder import constants
 from lcbuilder.LcBuild import LcBuild
 from lcbuilder.constants import CUTOUT_SIZE, LIGHTKURVE_CACHE_DIR
 from lcbuilder.objectinfo.MissionObjectInfo import MissionObjectInfo
@@ -28,9 +31,9 @@ class MissionLightcurveBuilder(LightcurveBuilder):
         author = object_info.author if object_info.author is not None else self.authors[mission]
         star_info = starinfo.StarInfo(sherlock_id, *self.star_catalogs[mission_prefix].catalog_info(id))
         logging.info("Downloading lightcurve files...")
-        sectors = None if object_info.sectors == 'all' or mission != "TESS" else object_info.sectors
-        campaigns = None if object_info.sectors == 'all' or mission != "K2" else object_info.sectors
-        quarters = None if object_info.sectors == 'all' or mission != "Kepler" else object_info.sectors
+        sectors = None if object_info.sectors == 'all' or mission != constants.MISSION_TESS else object_info.sectors
+        campaigns = None if object_info.sectors == 'all' or mission != constants.MISSION_K2 else object_info.sectors
+        quarters = None if object_info.sectors == 'all' or mission != constants.MISSION_KEPLER else object_info.sectors
         tokens = sectors if sectors is not None else campaigns if campaigns is not None else quarters
         tokens = tokens if tokens is not None else "all"
         apertures = {}
@@ -39,6 +42,9 @@ class MissionLightcurveBuilder(LightcurveBuilder):
             logging.info("There is data for Mission: %s, Year %.0f, Author: %s, ExpTime: %.0f",
                          tpf_search_result.mission[0], tpf_search_result.year[0], tpf_search_result.author[0],
                          tpf_search_result.exptime[0].value)
+        tpfs_dir = sherlock_dir + "/tpfs/"
+        if not os.path.exists(tpfs_dir):
+            os.mkdir(tpfs_dir)
         if object_info.apertures is None:
             lcf_search_results = lk.search_lightcurve(str(mission_id), mission=mission, cadence=cadence,
                                            sector=sectors, quarter=quarters,
@@ -56,6 +62,7 @@ class MissionLightcurveBuilder(LightcurveBuilder):
             lc = None
             matching_objects = []
             for tpf in tpfs:
+                shutil.copy(tpf.path, tpfs_dir + os.path.basename(tpf.path))
                 if mission_prefix == self.MISSION_ID_KEPLER:
                     sector = tpf.quarter
                 elif mission_prefix == self.MISSION_ID_TESS:
@@ -103,6 +110,7 @@ class MissionLightcurveBuilder(LightcurveBuilder):
             apertures = object_info.apertures
             lc = None
             for tpf in tpfs:
+                shutil.copy(tpf.path, tpfs_dir + os.path.basename(tpf.path))
                 if mission_prefix == self.MISSION_ID_KEPLER:
                     sector = tpf.quarter
                 elif mission_prefix == self.MISSION_ID_TESS:
